local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")

local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("DeviceManage")

function get_property(pk, dn, property_list)
    ret = utils_device.get_property_value(pk, dn, property_list)
    if (nil == ret) then
        return nil
    end
    list = {}
    for n, id in ipairs(property_list) do 
        item = {}
        item["iotId"] = ""
        item["batchId"] = ""
        item["attribute"] = id
        item["groud"] = ""
        item["value"] = ret[id]
        item["gmtModified"] = tostring(os.time())
        table.insert(list, item)
    end

    return list
end

if ("POST" == request_method) then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_boby_data()
    local pk = params["productKey"]
    local dn = params["deviceName"]
    local propertyIdentifier = params["propertyIdentifier"]

    utils.check_pk(pk)
    utils.check_dn(dn)

    if (nil == propertyIdentifier) then
        utils.error_exit(400, "invalid parameter")
    end

    -- transfer single string to table
    if ("string" == type(propertyIdentifier)) then
        propertyIdentifier = {propertyIdentifier}
    end

    ret = get_property(pk, dn, propertyIdentifier)

    res_raw = json.encode(ret)
    utils.error_exit(200, res_raw)
end

utils.error_exit(400, "unsupported request method")
